import { describe, it } from "vitest";
import {
	listResponse,
	singleResponse,
	createdResponse,
	tokenResponse,
	badRequestResponse,
	notFoundResponse,
	serverErrorResponse,
	unauthorizedResponse,
	forbiddenResponse,
} from "../responseBodies";
import { deepEqual, equal } from "assert";
import { createId } from "$lib/Id";

describe("responseBodies", () => {
	describe("the following response body functions return expected response", () => {
		it("listResponse", async () => {
			const expected = [{ some: "item", another: "thing" }];
			const { body, status } = await unwrapResponse(listResponse, expected);

			deepEqual(body, { items: expected });
			equal(status, 200);
		});

		it("singleResponse", async () => {
			const expected = { some: "item" };
			const { body, status } = await unwrapResponse(singleResponse, expected);

			deepEqual(body, { item: expected });
			equal(status, 200);
		});

		it("createdResponse", async () => {
			const expectedId = createId();
			const { body, status } = await unwrapResponse(createdResponse, expectedId);

			deepEqual(body, { item: expectedId });
			equal(status, 201);
		});

		it("tokenResponse", async () => {
			const token = "some-generated-token";
			const { body, status } = await unwrapResponse(tokenResponse, token);

			deepEqual(body, { access_token: token });
			equal(status, 200);
		});

		it("badRequestResponse", async () => {
			const errorString = "something bad happened";
			let { body, status } = await unwrapResponse(badRequestResponse, errorString);

			deepEqual(body, { error: errorString });
			equal(status, 400);

			const res = await unwrapResponse(badRequestResponse);
			body = res.body;
			status = res.status;

			deepEqual(body, { error: "Bad Request" });
			equal(status, 400);
		});

		it("notFoundResponse", async () => {
			const { body, status } = await unwrapResponse(notFoundResponse);

			deepEqual(body, { error: "Not Found" });
			equal(status, 404);
		});

		it("serverErrorResponse", async () => {
			const { body, status } = await unwrapResponse(serverErrorResponse);

			deepEqual(body, { error: "Unexpected Server Error" });
			equal(status, 500);
		});

		it("unauthorizedResponse", async () => {
			const { body, status } = await unwrapResponse(unauthorizedResponse);

			deepEqual(body, { error: "Unauthorized" });
			equal(status, 401);
		});

		it("forbiddenResponse", async () => {
			const { body, status } = await unwrapResponse(forbiddenResponse);

			deepEqual(body, { error: "Forbidden" });
			equal(status, 403);
		});
	});
});

async function unwrapResponse(fn: Function, body?: unknown) {
	const res = fn(body);
	return { status: res.status, body: await res.json() };
}
